"""**************************************************
This module has functions for extinction calculation
*****************************************************
"""
#print __doc__


import pdb
import scipy
from scipy import interpolate
import numpy as np
from . import get_filter


def extinction_in_filter(E,filter_family,filter_name,Model,R=None):
    """Description: Given a filter family and name, and E_{B-V}) calculate the extinction in magnitude A_{\lambda_eff}.
    The program works in the 0.1-2 micron range.
    The program is using the Cardelli, Clayton, Mathis (1989) or Allen models.
    for filters family and names see get_filter.py
    Input  :- E_{B-V}
            - filter_family (see available choices in get_filter)
            - filter_name (see available choices in get_filter)
            - Extinction law model:
                'A' Allen
                'C' Cardelli et al (default)
            - R=A_V/E_{B_V}, default is 3.08
    Output : A{\lambda_eff} (=A_V * model(lambda_eff, R_V))
    Tested : ?
         By : Maayane T. Soumagnac Nov 2016
        URL :
    Example: e=extinction.extinction_in_filter(E,'sdss','u','C')
    Reliable:  """
    L_micro=get_filter.filter_effective_wavelength(filter_family,filter_name)
    print('you gave the code the filter family and name name {0},{1} and it converted it into wavelenght {2} microns'.format(filter_family,filter_name,L_micro))
    #else:
    #   L_micro=L
    if R==None:
        R_V=3.08
    else:
        R_V=R
    if Model=='C':
        extinction_model_l=a_lambda_cardelli(L_micro,R_V)
    elif Model=='A':
        extinction_model_l=a_lambda_allen(L_micro)
    else: print("Unknown model, choose between 'A' (Allen) and 'C' (Cardelli)")
    A_lambda=R_V*E*extinction_model_l
    return A_lambda

def extinction_in_single_wavelength(E,L,Model,R=None):
    """Description: Given a unique wavelength, or a list of wavelengths, calculates the extinction in magnitude A_{\lambda}.
    The program works in the 0.1-2 micron range.
    The program is using the Cardelli, Clayton, Mathis (1989) or Allen models.
    for filters family and names see get_filter.py
    Input  :- E_{B-V}
            - wavelength: one wavelength or 1-D numpy array of wavelengths
            - Extinction law model:
                'A' Allen
                'C' Cardelli et al (default)
            - R=A_V/E_{B_V}, default is 3.08
    Output : A{\lambda} (=A_V * model(lambda, R_V)), the size of wavelengths
    Tested : ?
         By : Maayane T. Soumagnac Nov 2016
        URL :
    Example: extinction.extinction_in_wavelength(E,W,'C')
    Reliable:  """
    #L_micro=get_filter.filter_effective_wavelength(filter_family,filter_name)
    #print 'you gave the code the filter family and name name {0},{1} and it converted it into wavelenght {2} microns'.format(filter_family,filter_name,L_micro)
    #else:
    L_micro=L
    if R==None:
        R_V=3.08
    else:
        R_V=R
    if Model=='C':
        extinction_model_l=a_lambda_cardelli(L_micro,R_V)
    elif Model=='A':
        extinction_model_l=a_lambda_allen(L_micro)
    else: print("Unknown model, choose between 'A' (Allen) and 'C' (Cardelli)")
    A_lambda=R_V*E*extinction_model_l
    return A_lambda

def extinction_in_array_of_wavelength(E,wavelength,Model,R=None):
    """Description: Given a unique wavelength, or a list of wavelengths, calculates the extinction in magnitude A_{\lambda}.
    The program works in the 0.1-2 micron range.
    The program is using the Cardelli, Clayton, Mathis (1989) or Allen models.
    for filters family and names see get_filter.py
    Input  :- E_{B-V}
            - wavelength: one wavelength or 1-D numpy array of wavelengths, IN MICRONS
            - Extinction law model:
                'A' Allen
                'C' Cardelli et al (default)
            - R=A_V/E_{B_V}, default is 3.08
    Output : A{\lambda} (=A_V * model(lambda, R_V)), the size of wavelengths
    Tested : ?
         By : Maayane T. Soumagnac Nov 2016
        URL :
    Example: extinction.extinction_in_wavelength(E,W,'C')
    Reliable:  """
    #L_micro=get_filter.filter_effective_wavelength(filter_family,filter_name)
    #print 'you gave the code the filter family and name name {0},{1} and it converted it into wavelenght {2} microns'.format(filter_family,filter_name,L_micro)
    if R==None:
        R_V=3.08
    else:
        R_V=R
    if Model=='C':
        extinction_model_l=a_lambda_cardelli(wavelength,R_V)
    elif Model=='A':
        extinction_model_l=a_lambda_allen(wavelength)
    else: print("Unknown model, choose between 'A' (Allen) and 'C' (Cardelli)")
    #print R_V
    #print E
    A_lambda=R_V*E*extinction_model_l
    return A_lambda

def a_lambda_allen(W):
    Lambda = [2.0,1.0,0.9,0.67,0.553,0.50,0.44,0.40,0.365,0.333,0.285,0.250,0.222,0.200,0.167,0.143,0.125,0.111,0.100]
    A_Lam = [0.11,0.38,0.46,0.74,1.00,1.13,1.32,1.45,1.58,1.69,1.97,2.30,2.9,2.8,2.7,3.0,3.3,3.7,4.2]
    if np.max(W)>=2.0:
        print('You want the extinction for wavelength higher than 2 microns, try the cardelli model instead')
        pdb.set_trace()
    A_lambda_over_A_V=scipy.interpolate.interp1d(Lambda,A_Lam)(W)
    return A_lambda_over_A_V

def a_lambda_cardelli(W,R=None):
    """Description: inspired by Eran's function of the same name.
    Input  : wavelength in microns
    Output : numpy array of A(lambda), the size of wavelengths.
    Tested : ?
         By : Maayane T. Soumagnac Nov 2016
        URL :
    Example:
    Reliable:  """
    #print('Wavelengths in microns are',W)
    x = 1. / W
    if  isinstance(x, float)==True:
        x = np.array([x])
    y = x - 1.82
    #print np.shape(x)
    #print np.shape(y)
    a = np.zeros(np.shape(x))
    b = np.zeros(np.shape(x))
    A_lambda_over_A_V = np.zeros(np.shape(x))
    if R==None:
        R=3.08 # default average milkey way R
        #infrared
    for i,j in enumerate(x):
    #a[np.logical_and(x>=0.3,x<=1.1)]=0.574*x**
        if (j>=0.3 and j<=1.1):
            #print 'I am in the IR'
            a[i]=0.574*j**1.61
            b[i]=-0.527*j**1.61
        elif (j>=1.1 and j<=3.3):
            #print 'I am in the optical/NIR'
            a[i]=1 + 0.17699*y[i] - 0.50447*y[i]**2 - 0.02427*y[i]**3 + 0.72085*y[i]**4 + 0.01979*y[i]**5 - 0.77530*y[i]**6 + 0.32999*y[i]**7
            b[i]=1.41338*y[i] + 2.28305*y[i]**2 + 1.07233*y[i]**3 - 5.38434*y[i]**4 - 0.62251*y[i]**5 + 5.30260*y[i]**6 - 2.09002*y[i]**7
        elif (j >= 3.3 and j <= 8):
            if (j >= 5.9):
                #print 'I am in the UV'
                Fa = -0.04473*(j - 5.9)**2 - 0.009779*(j - 5.9)**3
                Fb = 0.2130*(j - 5.9)**2 + 0.1207*(j - 5.9)**3
            else:
                Fa = 0
                Fb = 0
            a[i] = 1.752 - 0.316*j- 0.104/((j - 4.67)**2 + 0.341) + Fa
            b[i] = -3.090 + 1.825*j + 1.206/((j - 4.62)**2 + 0.263) + Fb
        elif (j>=8 and j<=10):
            #print 'I am in the farUV'
            a[i] = -1.073 - 0.628*(j - 8.) + 0.137*(j- 8.)**2 - 0.070*(j - 8.)**3
            b[i] = 13.670 + 4.257*(j - 8.) - 0.420*(j - 8.)**2 + 0.374*(j- 8.)**3
        else:
            print('Illegal wavelength, should be in range 0.1-3.33 micron')
            pdb.set_trace()
        A_lambda_over_A_V[i]=a[i]+b[i]/R
    return A_lambda_over_A_V

def correct_obs_flux_for_extinction(observed_flux,Ebv,Model=None,R=None):
    """Description: Given an observed flux and an extinction Ebv, correct for the extinction, by doing
    f_true=f_obs*10^(0.4*A) (since, by definition of A, mag_obs=mag_true+A), with A calculated as in extinction_in_array_of_wavelength
    Input  :- flux (numpy array [wavelength in microns, flux] or [wavelength in microns, flux, errors])
            - extinction E_bv
            - optionnal: R (default is 3.08)
            - optionnal: Extinction law model:
                'A' Allen
                'C' Cardelli et al (default)
    Output : observed_flux (2-N numpy array same size as flux)
    Tested : ?
         By : Maayane T. Soumagnac Nov 2016
        URL :
    Example: e=
    Reliable:  """
    #print 'the extinction Ebv is {0}'.format(Ebv)
    if  isinstance(observed_flux, np.ndarray)==True:
        tens=np.zeros(np.shape(observed_flux)[0])
        tens[:]=10.
        corrected_flux=np.zeros(np.shape(observed_flux))
        if Model==None:
            if R==None:
                A=extinction_in_array_of_wavelength(Ebv,observed_flux[:,0],'C')
            else:
                A=extinction_in_array_of_wavelength(Ebv,observed_flux[:,0],'C',R)
        else:
            if R==None:
                A=extinction_in_array_of_wavelength(Ebv,observed_flux[:,0],Model)
            else:
                A=extinction_in_array_of_wavelength(Ebv,observed_flux[:,0],Model,R)
        corrected_flux[:,1]=np.multiply(observed_flux[:,1],np.power(tens,A*0.4))
        if np.shape(observed_flux)[1]>2:#if there are errors
            print('I am correcting the errors for extinction too')
            #pdb.set_trace()
            corrected_flux[:, 2] = np.multiply(observed_flux[:, 2], np.power(tens, A * 0.4))
        #print observed_flux[:,1]
        #print np.power(tens,A*0.4)
        #print np.multiply(observed_flux[:,1],np.power(tens,A*0.4))
        #print 'corrected_flux',corrected_flux[:,1]
        corrected_flux[:,0]=observed_flux[:,0]
    else:
        print('flux is an unknown data type')
        pdb.set_trace()
    return corrected_flux

def correct_obs_mag_for_extinction(observed_mag,Ebv,Model=None,R=None):
    """Description: Given an observed flux and an extinction Ebv, correct for the extinction, by doing
    mag_obs=mag_true+A, with A calculated as in extinction_in_array_of_wavelength
    Input  :- mag (numpy array [wavelength in microns, mag] or [wavelength in microns, mag, errors])
            - extinction E_bv
            - optionnal: R (default is 3.08)
            - optionnal: Extinction law model:
                'A' Allen
                'C' Cardelli et al (default)
    Output : observed_mag (2-N numpy array same size as flux)
    Tested : ?
         By : Maayane T. Soumagnac Nov 2016
        URL :
    Example: e=
    Reliable:  """
    #print 'the extinction Ebv is {0}'.format(Ebv)
    if  isinstance(observed_mag, np.ndarray)==True:
        tens=np.zeros(np.shape(observed_mag)[0])
        tens[:]=10.
        corrected_mag=np.zeros(np.shape(observed_mag))
        if Model==None:
            if R==None:
                A=extinction_in_array_of_wavelength(Ebv,observed_mag[:,0],'C')
            else:
                A=extinction_in_array_of_wavelength(Ebv,observed_mag[:,0],'C',R)
        else:
            if R==None:
                A=extinction_in_array_of_wavelength(Ebv,observed_mag[:,0],Model)
            else:
                A=extinction_in_array_of_wavelength(Ebv,observed_mag[:,0],Model,R)
        corrected_mag[:,1]=observed_mag[:,1]-A
        if np.shape(observed_mag)[1]>2:#if there are errors
            print('I am not correcting the errors since we are in mag')
            #pdb.set_trace()
            corrected_mag[:, 2] = observed_mag[:, 2]
        #print observed_mag[:,1]
        #print np.power(tens,A*0.4)
        #print np.multiply(observed_mag[:,1],np.power(tens,A*0.4))
        #print 'corrected_mag',corrected_mag[:,1]
        corrected_mag[:,0]=observed_mag[:,0]
    else:
        print('mag is an unknown data type')
        pdb.set_trace()
    return corrected_mag

def apply_extinction_to_theoretical_flux(theoretical_flux,Ebv,Model=None,R=None):
    """Description: Given a theorectical flux f_true and an extinction Ebv, apply extinction to simulate the observed flux, by doing
    f_obs=f_th*10^(-0.4*A) (since, by definition of A, mag_obs=mag_th+A), with A calculated as in extinction_in_array_of_wavelength
    Input  :- theoretical_flux (numpy array [wavelength in microns, flux])
            - extinction E_bv
            - optionnal: R (default is 3.08)
            - optionnal: Extinction law model:
                'A' Allen
                'C' Cardelli et al (default)
    Output : theoretical_flux (2-N numpy array same size as flux)
    Tested : ?
         By : Maayane T. Soumagnac Nov 2016
        URL :
    Example: e=
    Reliable:  """
    #print('the extinction Ebv is {0}'.format(Ebv))
    if  isinstance(theoretical_flux, np.ndarray)==True:
        tens=np.zeros(np.shape(theoretical_flux)[0])
        #print 'the shape of theoretica flux is {0}'.format(np.shape(theoretical_flux))
        #print 'the shape of theoretica tens is {0}'.format(np.shape(tens))
        tens[:]=10.
        corrected_flux=np.zeros(np.shape(theoretical_flux))
        if Model==None:
            if R==None:
                A=extinction_in_array_of_wavelength(Ebv,theoretical_flux[:,0],'C')
            else:
                A=extinction_in_array_of_wavelength(Ebv,theoretical_flux[:,0],'C',R)
        else:
            if R==None:
                A=extinction_in_array_of_wavelength(Ebv,theoretical_flux[:,0],Model)
            else:
                A=extinction_in_array_of_wavelength(Ebv,theoretical_flux[:,0],Model,R)
        #print np.shape(corrected_flux)
        #print np.shape(tens)
        corrected_flux[:,1]=np.multiply(theoretical_flux[:,1],np.power(tens,-A*0.4))
        #print('theretical flux',theoretical_flux[:,1])
        #print np.power(tens,A*0.4)
        #print np.multiply(theoretical_flux[:,1],np.power(tens,A*0.4))
        #print('corrected_flux',corrected_flux[:,1])
        corrected_flux[:,0]=theoretical_flux[:,0]
    else:
        print('flux is an unknown data type')
        pdb.set_trace()
    #if plot==True:
    #pylab.figure()
    #pylab.plot(theoretical_flux[:,0],theoretical_flux[:,1],label='spectrum before')
    #pylab.plot(corrected_flux[:, 0], corrected_flux[:, 1], label='spectrum after')
    #pylab.legend()
    #pylab.show()
    return corrected_flux
















#! //anaconda/bin/python



import pdb
import numpy as np
import pyphot

# It would make sens to reorganize this as a class Filter, with attributes: family_name, name,
# and methods effective wavelength, tranmission curve, filter half width, see for example Eran's get_filter

def filter_effective_wavelength(filter_family,filtername):
    """Description: inspired by Eran's finction of the same name. Given a filter name, finds the effective wavelength in microns
    Input  :-Filter family among:
                Johnson, sdss, poss
            -Filter name among:
                Johnson: 'U','B','V','R','I','J','H','K' or in minuscules
                sdss: 'u','g','r','i','z'
                poss: 'O','E'
    Output  :-effective wavelength in microns (kind of a hybrid between wavelength-weighted average and frequency-weighted average. See reference)
    Reference: Fukugita et al. 1996, Table 2a.
    Tested : ?
    By : Maayane T. Soumagnac Nov 2016
    URL :
    Example:
    Reliable:  """
    if filter_family.lower()=='sdss':
        if filtername.lower()=='u':
            W=0.3557
        elif filtername.lower()=='g':
            W=0.4825
        elif filtername.lower()=='r':
            W=0.6261
        elif filtername.lower()=='i':
            W=0.7672
        elif filtername.lower()=='z':
            W=0.9097
        else: print('unknown SDSS filter name')
    elif filter_family.lower()=='johnson':
        if filtername.lower()=='u':
            W=0.367#?
        elif filtername.lower()=='b':
            W=0.436#?
        elif filtername.lower()=='v':
            W=0.545
        elif filtername.lower()=='r':
            W=0.638
        elif filtername.lower()=='i':
            W=0.797
        elif filtername.lower()=='j':
            W=1.220
        elif filtername.lower()=='h':
            W=1.630
        elif filtername.lower()=='k':
            W=2.190
        else: print('unknown Johnson filter name')
    elif filter_family.lower()=='poss':
        if filtername.lower()=='o':
            W=0.420
        elif filtername.lower()=='e':
            W=0.640
    else:
        print('unknown family filter name. You can choose among [sdss, johnson, poss] and it is case insensitive')
        pdb.set_trace()
    return W


def make_filter_object(Filter_vector,filters_directory,central=True,verbose=False):
    """Description: from a filter vector where each element is a couple [filter family,filter name], create a filter object P as in pyphoy
            Input  :- a filter vector: a numpy array where each element is a tuple of strings [filter family,filter name]
                    - central. If true, gives pyphot .cl wavelength, which corresponds to Eran's AstFilter.get('family','band').eff_wl
                                else, gives phyphot .eff wavelength, which I am not sure what it is..
            Output :- filter vector, with additional column : P object
                    - a np.array with the effective wavelength of each filter
            with the corresponding data
            Tested : ?
                 By : Maayane T. Soumagnac Nov 2016
                URL :
            Example:Filter_vector = np.empty([2, 2], dtype=object)
                    Filter_vector[0] = [str('GALEX'), str('GALEX_NUV')]
                    Filter_vector[1]=[str('ptf_p48'),str('r')]
                    [P, wav]=make_filter_object(Filter_vector)
            Reliable:  """
    wavelength_filter_effective = dict()#np.empty(np.shape(Filter_vector)[0])
    wavelength_filter_central=dict()
    wavelength_filter_pivot = dict()
    P_vector=dict()
    if  isinstance(Filter_vector, (list,)):
        P_vector['filter_family']=Filter_vector[0][0]
        P_vector['filter_name']=Filter_vector[0][1]
        P_vector['filter_object'] = []
    else:
        P_vector['filter_family']=Filter_vector[:, 0]
        P_vector['filter_name']=Filter_vector[:,1]
        P_vector['filter_object'] = []
    #print("P_vector['filter_object'] is",P_vector['filter_object'])
    #pdb.set_trace()
    for i, j in enumerate(Filter_vector):
        '''
        if j[0].lower() == 'ptf_p48':
            #print(j[1])
            if j[1].lower() == 'g_p48_ptf':
                if verbose == True:
                    print('You gave the G filter of the PTF_P48 family')
                #pdb.set_trace()
                Transmission = np.genfromtxt(
                    '/Users/maayanesoumagnac/Maayane_Astro_python_library/data/filters/PTF_G.rtf', delimiter=None)
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='PTF_P48_G', dtype='photon', unit='Angstrom')
            elif j[1].lower() == 'r_p48_ptf':
                if verbose==True:
                    print('You gave the R filter of the PTF_P48 family')
                Transmission = np.genfromtxt(
                    '/Users/maayanesoumagnac/Maayane_Astro_python_library/data/filters/P48_R_T.rtf', delimiter=None)
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='PTF_P48_R', dtype='photon', unit='Angstrom')
        '''
        if j[0].lower() == 'ztf_p48':
            #print(j[1])
            if j[1].lower() == 'g_p48':
                if verbose == True:
                    print('You gave the G filter of the ztf_P48 family')
                #pdb.set_trace()
                Transmission = np.genfromtxt(filters_directory+'/ZTF_g_fromgithub_AA.txt', delimiter=None)
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='ZTF_P48_G', dtype='photon', unit='Angstrom')
            elif j[1].lower() == 'r_p48':
                if verbose==True:
                    print('You gave the R filter of the ZTF_P48 family')
                Transmission = np.genfromtxt(filters_directory+'/ZTF_r_fromgithub_AA.txt', delimiter=None)
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='ZTF_P48_R', dtype='photon', unit='Angstrom')
            elif j[1].lower() == 'i_p48':
                if verbose==True:
                    print('You gave the I filter of the ZTF_P48 family')
                Transmission = np.genfromtxt(filters_directory+'/ZTF_i_fromgithub_AA_reorder.txt', delimiter=None)
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='ZTF_P48_I', dtype='photon', unit='Angstrom')
        elif j[0].lower() == 'swift':
            if j[1].lower() == 'uvw1':
                if verbose == True:
                    print('You gave the uvw1 filter of the swift family')
                #pdb.set_trace()
                Transmission = np.genfromtxt(filters_directory+'/Swift_UVW1.rtf', delimiter=None)
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='Swift_UVW1', dtype='photon',
                                  unit='Angstrom')
        #elif j[0].lower() == 'swift':
            elif j[1].lower() == 'uvw2':
                if verbose == True:
                    print('You gave the uvw2 filter of the swift family')
                #pdb.set_trace()
                Transmission = np.genfromtxt(
                    filters_directory+'/Swift_UVW2.rtf', delimiter=None)
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='Swift_UVW2', dtype='photon',
                                  unit='Angstrom')
        #elif j[0].lower() == 'swift':
            elif j[1].lower() == 'uvm2':
                if verbose == True:
                    print('You gave the uvm2 filter of the swift family')
                #pdb.set_trace()
                Transmission = np.genfromtxt(
                    filters_directory+'/Swift_UVM2.rtf', delimiter=None)
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='Swift_UVM2', dtype='photon',
                                  unit='Angstrom')
            elif j[1].lower() == 'u_swift':
                if verbose == True:
                    print('You gave the u filter of the swift family')
                #pdb.set_trace()
                Transmission = np.genfromtxt(
                    filters_directory+'/Swift_u.rtf', delimiter=None)
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='Swift_u', dtype='photon',
                                  unit='Angstrom')
            elif j[1].lower() == 'v_swift':
                if verbose == True:
                    print('You gave the v filter of the swift family')
                #pdb.set_trace()
                Transmission = np.genfromtxt(
                    filters_directory+'/Swift_V.rtf', delimiter=None)
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='Swift_V', dtype='photon',
                                  unit='Angstrom')
            elif j[1].lower() == 'b_swift':
                if verbose == True:
                    print('You gave the b filter of the swift family')
                #pdb.set_trace()
                Transmission = np.genfromtxt(
                    filters_directory+'/Swift_B.rtf', delimiter=None)
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='Swift_B', dtype='photon',
                                  unit='Angstrom')
        elif j[0].lower() == 'galex':
            if j[1].lower() == 'galex_nuv':
                if verbose == True:
                    print('You gave the nuv filter of the galex family')
                #pdb.set_trace()
                Transmission = np.genfromtxt(
                    filters_directory+'/GALEX_NUV.dat', delimiter=None)
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='galex_nuv', dtype='photon',
                                  unit='Angstrom')
        elif j[0].lower() == 'sdss':
            if verbose == True:
                print('You gave the sdss family')
            #print(j[1].lower())
            if j[1].lower() == 'r_sdss':
                if verbose == True:
                    print('You gave the r filter of the sdss family')
                #pdb.set_trace()
                Transmission = np.genfromtxt(
                    filters_directory+'/SDSS_r.txt', delimiter=',')
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='r_sdss', dtype='photon',
                                  unit='Angstrom')
            elif j[1].lower() == 'g_sdss':
                if verbose == True:
                    print('You gave the g filter of the sdss family')
                #pdb.set_trace()
                Transmission = np.genfromtxt(
                    filters_directory+'/SDSS_g.txt', delimiter=',')
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='g_sdss', dtype='photon',
                                  unit='Angstrom')
            elif j[1].lower() == 'i_sdss':
                if verbose == True:
                    print('You gave the i filter of the sdss family')
                #pdb.set_trace()
                Transmission = np.genfromtxt(
                    filters_directory+'/SDSS_i.txt', delimiter=',')
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='i_sdss', dtype='photon',
                                  unit='Angstrom')
            elif j[1].lower() == 'u_sdss':
                if verbose == True:
                    print('You gave the u filter of the sdss family')
                #pdb.set_trace()
                Transmission = np.genfromtxt(
                    filters_directory+'/SDSS_u.txt', delimiter=',')
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='u_sdss', dtype='photon',
                                  unit='Angstrom')

            elif j[1].lower() == 'z_sdss':
                if verbose == True:
                    print('You gave the z filter of the sdss family')
                #pdb.set_trace()
                Transmission = np.genfromtxt(filters_directory+'/SDSS_z.txt', delimiter=',')
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='z_sdss', dtype='photon',
                                  unit='Angstrom')

        elif j[0].lower() == '2mass':
            if verbose == True:
                print('You gave the 2MASS family')
                print(j[1].lower())
            if j[1].lower() == 'j_2mass':
                if verbose == True:
                    print('You gave the J filter of the 2MASS family')
            # pdb.set_trace()
                Transmission = np.genfromtxt(filters_directory+'/2MASS_J.txt',
                                         delimiter=',')
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='2MASS_J', dtype='photon',
                              unit='Angstrom')
            elif j[1].lower() == 'h_2mass':
                if verbose == True:
                    print('You gave the h filter of the 2MASS family')
            # pdb.set_trace()
                Transmission = np.genfromtxt(filters_directory+'/2MASS_H.txt',
                                         delimiter=',')
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='h_2mass', dtype='photon',
                              unit='Angstrom')
            elif j[1].lower() == 'k_2mass':
                if verbose == True:
                    print('You gave the k filter of the 2MASS family')
                    # pdb.set_trace()
                Transmission = np.genfromtxt(
                    filters_directory+'/2MASS_K.txt',
                    delimiter=',')
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='k_2mass', dtype='photon',
                                  unit='Angstrom')
        elif j[0].lower() == 'cousin':
            if verbose == True:
                print('You gave the cousin family')
                #print(j[1].lower())
            if j[1].lower() == 'i_cousin':
                if verbose == True:
                    print('You gave the i filter of the cousin family')
            # pdb.set_trace()
                Transmission = np.genfromtxt(filters_directory+'/cousin_i.txt',
                                         delimiter=',')
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='i_counsin', dtype='photon',
                              unit='Angstrom')
            elif j[1].lower() == 'r_cousin':
                if verbose == True:
                    print('You gave the r filter of the cousin family')
                Transmission = np.genfromtxt(filters_directory+'/cousin_r.txt',
                                         delimiter=',')
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='r_cousin', dtype='photon',
                              unit='Angstrom')
        elif j[0].lower() == 'johnson':
            if verbose == True:
                print('You gave the johnson family')
                print(j[1].lower())
            if j[1].lower() == 'u_johnson':
                if verbose == True:
                    print('You gave the u filter of the johnson family')
            # pdb.set_trace()
                Transmission = np.genfromtxt(filters_directory+'/johnson_u.txt',
                                         delimiter=',')
                #print('the shape of transission is',Transmission)
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='u_johnson', dtype='photon',
                              unit='Angstrom')
            elif j[1].lower() == 'b_johnson':
                if verbose == True:
                    print('You gave the b filter of the johnson family')
            # pdb.set_trace()
                Transmission = np.genfromtxt(filters_directory+'/johnson_b.txt',
                                         delimiter=',')
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='b_johnson', dtype='photon',
                              unit='Angstrom')
            elif j[1].lower() == 'v_johnson':
                if verbose == True:
                    print('You gave the v filter of the johnson family')
                    # pdb.set_trace()
                Transmission = np.genfromtxt(
                    filters_directory+'/johnson_v.txt',
                    delimiter=',')
                P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='v_johnson', dtype='photon',
                                  unit='Angstrom')

        else:
            print('I HAVE NOT RECOGNIZE THE FILTER')
            lib = pyphot.get_library()
            f = lib.find(j[0].lower())  # filter family
            if verbose == True:
                for name in f:
                    lib[name].info(show_zeropoints=True)
            P = lib[j[1]]  # filter name
        #print(P_vector)
        #print(P)
        P_vector['filter_object'].append(P)
        wavelength_filter_effective[P_vector['filter_name'][i]]=P.leff.item()
        wavelength_filter_central[P_vector['filter_name'][i]]=P.cl.item()
        if isinstance(Filter_vector, (list,)):
            P_vector['filter_object']=P_vector['filter_object'][0]
    if central==True:
        return P_vector, wavelength_filter_central#same as Eran eff_wl
    else:
        return P_vector,wavelength_filter_effective

        #return 'hi'


from PhotoFit import PhotoFit_fun
import params_test as params
import os
import pylab

########## Definition of the output file ##########
output_file_mcmc=params.output_file_mcmc
output_file_linear=params.output_file_linear
output_file_interpolation=params.output_file_interpolation

if params.mcmc==True:
    output=output_file_mcmc
else:
    output=output_file_linear
if os.path.exists(output):
    print('the output directory, '+output + ' exists already')
else:
    os.mkdir(output)

########################################### Read and process the photometric data ############################################

Best=PhotoFit_fun.calculate_T_and_R_in_time(data_file=params.data_file,dates_file=params.dates_file,already_run_interp_errors_from_param=params.already_run_interp_errors,
                                            already_run_mcmc=params.already_run_mcmc,already_run_matrix=params.already_run_matrix,
                                            num_iterations=params.num_iterations,show_underlying_plots=True,verbose=True,redshift=params.z,
                                            distance_modulus=params.distance_modulus,explosion_date=params.explosion_date,EBV=params.EBV,
                                            output=output,filters_directory=params.filters_directory,mcmc=params.mcmc,
                                            output_file_interpolation=params.output_file_interpolation,
                                            lower_limit_on_flux=params.lower_limit_on_flux,csm=params.csm,already_run_fit=params.already_run_fit,num_steps=params.num_steps,nwalkers=params.nwalkers)
PhotoFit_fun.plot_T_and_R_in_time(Best,data_compare=params.data_compare,compare=False,label_comparision='PTF 13dqy',output=output)
pylab.show()

#PhotoFit_fun.plot_L_in_time(Best,data_file=params.data_file,lower_limit_on_flux=params.lower_limit_on_flux,dates_file=params.dates_file,error_lum_ran=False,explosion_date=params.explosion_date,output=output,mcmc=params.mcmc,output_file_interpolation=params.output_file_interpolation)

PhotoFit_fun.plot_SEDs(Best,already_interpolated=True,data_file=params.data_file,lower_limit_on_flux=params.lower_limit_on_flux,dates_file=params.dates_file,already_run_interp_errors_from_params=params.already_run_interp_errors, number_of_plot=9,redshift=params.z,distance_modulus=params.distance_modulus,explosion_date=params.explosion_date,output=output,filters_directory=params.filters_directory,output_file_interpolation=params.output_file_interpolation,EBV=params.EBV)




#! //anaconda/bin/python



import numpy as np
import sys
import os
import numpy as np
from . import black_body_flux_density
from . import calc_black_body_flux_filters
import pylab
from . import fitting_tools
from . import distances_conversions
from . import energy_conversions
import math
import shutil
import logging
import pyphot
from . import get_filter
import pdb
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

def fit_black_body_flux_filters(Spectrum,TempVec=None,num_temp_iterations=None,distance=None,Ebv=None,ndof=None,uncertainties=None,show_plot=False,output_file_path=None,show_mag_AB=True,z=0,path_to_txt_file=None,filters_directory=None):
	"""Description: Fit a black-body spectrum alpha*F_bb to an observed spectrum, given as fuxes through a set of filters.
	Input  :- Observed spectrum [Filter familly, filter_name, Flux]. flux in [m,erg/cm^/s/A]
			- Vector of temperatures to test [K].If empty matrix the use default. Default is logspace(3,6,num_temp_iterations). For each of these temperature, the code fits A*Model(T) to the data. then the best temperature is chosen
			- num_temp_iterations: optionnal number of temperatures in the defaukte TempVec. Default is 100
			- Optional distance [pc], default is 10pc.
			- Extinction correction (E_{B-V}) to apply to the spectrum before the fit. Default is 0.
			The correction is applied as: f_true=f_obs*10^(+0.4*A), where A is defined as mag_obs=mag_true+A.
			SInce the extinction is wavelength dependant, the wavelength used to correct the spectrum is the central wavelength of each filter.
			- show_plot: if set to True, show the plot. create it in anycase.
			- output_file_path: optional output file name, (if you run this function in a loop and do not want the output directories to be removed), default is outputs_from_fit_black_body_flux_filters_function
			-z: redshift to correct the spectrum for
	Output :- numpy array with the following fields:
				*temperature
				*for this temperature, best fit multiplication factor alpha. alpha is (R/d)^2, and therefore gives you the radius if you know the distance.
	 		(see my lecture notes in observationnal astronomy for an explanation of alpha).
	 			*for this temperature, value of best fit chi2
	 			*for this temperature, value of the radius
	 		-value of best temperature
	 		-value of best temperature index in Xi_array
	 		-value of best temperature coeff1
	 		-value of best radius
	 		-value of best luminosity
	Plots and output files: -plot best fit in .outputs_from_fit_black_body_flux_filters_function/fit_result.png in flux
							- if show_mag_AB is True, same in mag AB
							-plot chi2, coeff, and R in outputs_from_fit_black_body_flux_filters_function/fit_grid_results.png
							-save sorted_Xi_array in outputs_from_fit_black_body_flux_filters_function/Sorted_fir_results.txt
	Tested : ?
	    By : Maayane T. Soumagnac Nov 2016
	   URL :
	Example: [Xi_array,best_temp,index_min,best_coeff,best_radius]=fit_black_body.fit_black_body_spec(black_body_with_errors,distance=1.,show_plot=True)
	Reliable:
	 TO DO: give an option for the speed: decrease the length of TempVec, give the spec units as options,
	 and change the location /Users/maayanesoumagnac/Maayane_Astro_python_library/data to the user one"""
	#print 'the spectrum is',Spectrum
	#pdb.set_trace()
	if output_file_path is None:
		if os.path.exists('./utputs_from_fit_black_body_flux_filters_function'):
			logger.info('output_path/txt_files did exist')
			#shutil.rmtree('./outputs_from_fit_black_body_flux_filters_function')
		else:
			logger.info('the output file file did not exist yet. I am creating it now')
			os.makedirs('./outputs_from_fit_black_body_flux_filters_function')
		output_file='./outputs_from_fit_black_body_flux_filters_function'
	else:
		if os.path.exists(str(output_file_path)):
			logger.info('output_path did exist')
			#shutil.rmtree(output_file_path)
		else:
			logger.info('the output file file did not exist yet. I am creating it now')
			os.makedirs(output_file_path)
		output_file=output_file_path

	#******************** convert waveunits into meters ***********************
	Spectrum_good_units=np.zeros(np.shape(Spectrum))
	Filter_vector=np.empty([np.shape(Spectrum)[0], 2],dtype=object)
	for i,j in enumerate(Spectrum):
		Filter_vector[i]=[str(Spectrum[i,0]),str(Spectrum[i,1])]
	wavelengths_min=np.zeros(np.shape(Filter_vector)[0])
	wavelengths_max=np.zeros(np.shape(Filter_vector)[0])
	central_wavelength=np.zeros(np.shape(Filter_vector)[0])
	mags=np.zeros(np.shape(Filter_vector)[0])
	mags_correct = np.zeros(np.shape(Filter_vector)[0])
	lib = pyphot.get_library()
	#print('The filters library contains {0} filters'.format(len(lib)))
	#print('from which the following filters are used here:')
	print('Filter_vector:',Filter_vector)
	[P_vectorx, effective_wavelength] = get_filter.make_filter_object(Filter_vector,filters_directory=filters_directory)
	#print('the shape of Filter vector is', np.shape(Filter_vector)[0])
	P_vector = np.empty((np.shape(Filter_vector)[0], 3), dtype=object)
	#print(np.shape(P_vector))
	#print(P_vectorx['filter_family'])
	#print(np.shape(P_vectorx['filter_family'][:]))
	P_vector[:, 0] = P_vectorx['filter_family'][:]
	P_vector[:, 1] = P_vectorx['filter_name'][:]
	P_vector[:, 2] = P_vectorx['filter_object'][:]
	if np.shape(Spectrum)[1]>3:
		spectrum_flux = np.zeros((np.shape(Filter_vector)[0], 3))
	else:
		spectrum_flux = np.zeros((np.shape(Filter_vector)[0], 2))
	for i,s in enumerate(Filter_vector):
		#wavelengths_min[i]=np.min(P_vector[i,2].wavelength.magnitude)
		#wavelengths_max[i]=np.max(P_vector[i,2].wavelength.magnitude)
		#print(type(Spectrum[i,2]))
		mags[i] = -2.5 * np.log10(Spectrum[i, 2]) - P_vector[i,2].AB_zero_mag
		#spectrum_flux[i,0]=P_vector[i,2].leff.item()
		spectrum_flux[i, 0] = P_vector[i, 2].cl.item()#le eff_wl de Eran (auquel je veux pouvoir comparer) est le cl de pyphot
		spectrum_flux[i,1]=Spectrum[i,2]
	if np.shape(Spectrum)[1] > 3:
		spectrum_flux[i,2]=Spectrum[i,3]
	min_wavelength=np.min(wavelengths_min)
	max_wavelength=np.max(wavelengths_max)
	wavelengths=np.linspace(min_wavelength,max_wavelength,num=1000)
	#print('wavelengths are',wavelengths)
	#pdb.set_trace()
	wavelengths_in_meters=wavelengths*1e-10
	#print(P_vector)
	'''
	for i, s in enumerate(Filter_vector):

		if s[0].lower() == 'ptf_p48':
			if s[1].lower() == 'r':
				print 'You gave the R filter of the PTF_P48 family'
				Transmission = np.genfromtxt('/Users/maayanesoumagnac/Maayane_Astro_python_library/data/filters/P48_R_T.rtf', delimiter=None)
				P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='PTF_P48_R', dtype='photon',
								  unit='Angstrom')
		else:
			f = lib.find(s[0].lower())  # filter family
			for name_filter in f:
				lib[name_filter].info(show_zeropoints=True)
			P = lib[s[1]]  # filter name
		wavelengths_min[i]=np.min(P.wavelength.magnitude)
		wavelengths_max[i]=np.max(P.wavelength.magnitude)
		central_wavelength[i]=P.cl.item()
		print type(Spectrum[i,2])
		mags[i] = -2.5 * np.log10(Spectrum[i, 2]) - P.AB_zero_mag

	min_wavelength=np.min(wavelengths_min)
	max_wavelength=np.max(wavelengths_max)
	wavelengths=np.linspace(min_wavelength,max_wavelength,num=1000)
	wavelengths_in_meters=wavelengths*1e-10
	'''
	# ******************** Fit ************************
	if TempVec is None:
		if num_temp_iterations is None:
			print('I am exploring 100 temperatures')
			Temp=np.logspace(3.,6.,500)
		else:
			print('I am exploring {0} temperatures'.format(num_temp_iterations))
			Temp = np.logspace(3., 6., num_temp_iterations)
	if distance is None:
		dist=distances_conversions.pc_to_cm(10) # dist in cm
	else:
		Temp=TempVec
		dist=distances_conversions.pc_to_cm(distance)
	coeff1=np.zeros(np.shape(Temp))
	Xi_array=np.empty([np.shape(Temp)[0],4], dtype=object)
	if Ebv is None:
		Ebv=0
	#Spectrum_correct=np.empty(np.shape(Spectrum),dtype=object)
	#Spectrum_correct[:,0]=Spectrum[:,0]
	#Spectrum_correct[:,1]=Spectrum[:,1]
	#Spectrum_correct[:,2]=correct_spectrum.correct_spectrum_for_redshift_and_extinction(np.array(zip(central_wavelength, Spectrum[:,2])),comments="#",WaveUnits='A',Ebv=Ebv,z=z,show_plot=False,title=None,output_png_file=None)[:,1]
	'''
	if show_mag_AB==True:
			for i, s in enumerate(Filter_vector):
				if s[0].lower() == 'ptf_p48':
					if s[1].lower() == 'r':
						print 'You gave the R filter of the PTF_P48 family'
						Transmission = np.genfromtxt(
							'/Users/maayanesoumagnac/Maayane_Astro_python_library/data/filters/P48_R_T.rtf',
							delimiter=None)
						P = pyphot.Filter(Transmission[:, 0], Transmission[:, 1], name='PTF_P48_R', dtype='photon',
										  unit='Angstrom')
				else:
					P = lib[s[1]]  # filter name
				mags_correct[i] = -2.5 * np.log10(Spectrum_correct[i, 2]) - P.AB_zero_mag
	'''
	if uncertainties is None:
		for i, j in enumerate(Temp):
			#print(wavelengths_in_meters)
			#pdb.set_trace()
			print('*********************')
			print('i=',i)
			print('*********************')
			A = np.array(calc_black_body_flux_filters.calc_black_body_flux_filters(j, np.arange(1e-7, 3e-6, 1e-9), Filter_vector=Filter_vector,Radius=None, distance_pc=None, output_plot=False,show_plots=False,output_txt=False, lib=lib,z=z,Ebv=Ebv)[:, 3])
			matrix_solution = np.dot(1. / (np.dot(A.transpose(), A)), (np.dot(A.transpose(), spectrum_flux[:,1])))
			coeff1[i] = matrix_solution
			Xi_array[i, 2] = fitting_tools.objective_no_cov(
					coeff1[i] * calc_black_body_flux_filters.calc_black_body_flux_filters(j, np.arange(1e-7, 3e-6, 1e-9),
																						  Filter_vector=Filter_vector, Radius=None,
																						  distance_pc=None,
																						  output_txt=False,
																						  output_plot=False, lib=lib,z=z, Ebv=Ebv)[:, 3],spectrum_flux[:, 0:2]).chi_square_value()
			Xi_array[i, 0] = j
			Xi_array[i,1]=coeff1[i]
			if coeff1[i]<=0:
				Xi_array[i,3]=0
			else:
				Xi_array[i,3]= math.sqrt(coeff1[i]*dist**2)#radius in cm
	else:
		invcov=np.diag(np.power(uncertainties, -2))
		#print('invcov is',invcov)
		#print('shape is',np.shape(invcov))
		#pdb.set_trace()
		for i, j in enumerate(Temp):
			index = i  #
			print('*********************')
			print('i=',i)
			print('*********************')
			A = np.array(
				calc_black_body_flux_filters.calc_black_body_flux_filters(j, np.arange(1e-7, 3e-6, 1e-9), Filter_vector=Filter_vector,
																		  Radius=None, distance_pc=None, output_txt=False,
																		  output_plot=False, lib=lib,z=z,Ebv=Ebv,filters_directory=filters_directory)[:, 3])
			matrix_solution=np.dot(1./(np.dot(np.dot(A.transpose(),invcov),A)),(np.dot(np.dot(A.transpose(),invcov),spectrum_flux[:,1])))
			coeff1[i] = matrix_solution
			#print('coeff1[i] is',coeff1[i])
			#print('j is',j)
			Xi_array[i, 2] = fitting_tools.objective_with_cov(
					coeff1[i] * calc_black_body_flux_filters.calc_black_body_flux_filters(j, np.arange(1e-7, 3e-6, 1e-9),
																						  Filter_vector=Filter_vector, Radius=None,
																						  distance_pc=None,
																						  output_txt=False,
																						  output_plot=False, lib=lib,z=z,Ebv=Ebv,filters_directory=filters_directory)[:,
								3], spectrum_flux[:,0:2], invcov).chi_square_value()
			coeff1[i] = matrix_solution
			Xi_array[i, 0] = j
			Xi_array[i, 1] = coeff1[i]
			if coeff1[i]<=0:
				Xi_array[i,3]=0
			else:
				Xi_array[i,3]= math.sqrt(coeff1[i]*dist**2)#radius in cm
	#print('Xi_array[:,2] is',Xi_array[:,2])
	#pdb.set_trace()
	index_min=np.argmin(Xi_array[:,2])
	best_coeff1=coeff1[index_min]
	best_temp=Temp[index_min]
	best_radius=Xi_array[index_min,3]
	#mettre best radius en m: 4piR^2sigmaT^4
	best_luminosity=energy_conversions.convert_energy(4*math.pi*(best_radius*1e-2)**2*5.7e-8*best_temp**4,'J','erg') #en w
	sorted_Xi_array = Xi_array[Xi_array[:,2].argsort()]
	if distance is None:
		np.savetxt(output_file+'/Sorted_fit_result.txt', sorted_Xi_array,
				   header='best temperature, best coefficient A (in model A*blackbody), sorted chi2, best fit radius (cm) at distance 10pc ')
	else:
		np.savetxt(output_file+'/Sorted_fit_result.txt', sorted_Xi_array,
				   header='best temperature, best coefficient A (in model A*blackbody), sorted chi2, best fit radius (cm) at distance '+str(distance)+'pc')

	#******************** Plots ************************

	#Plot the fitting results
	pylab.figure()
	#print('effective wave',effective_wavelength)
	#print(Spectrum[:,2])
	print('spectrum_flux[:,0]',spectrum_flux[:,0])
	print('spectrum_flux[:,1]', spectrum_flux[:, 1])
	pylab.plot(spectrum_flux[:,0], spectrum_flux[:,1], 'mo',label=r'data')
	'''
	if Ebv!=0 and z!=0:
		pylab.plot(central_wavelength, Spectrum_correct[:, 2], 'bo', label=r'data corrected for extinction')
	if Ebv==0 and z!=0:
		pylab.plot(central_wavelength, Spectrum_correct[:, 2], 'bo', label=r'data corrected for redshift')
	if Ebv!=0 and z==0:
		pylab.plot(central_wavelength, Spectrum_correct[:, 2], 'bo', label=r'data corrected for extinction')
	'''
	#This is in case there are 2 param in model l1*black_body+l2: pylab.plot(Spectrum_corrected[:,0]*1e9,best_coeff1*black_body_flux_density.black_body_flux_density(best_temp, Spectrum_corrected[:, 0], 'P')[0][:,1]+best_coeff2,'-b',label='best fit')
	#print(best_coeff1*calc_black_body_flux_filters.calc_black_body_flux_filters(best_temp,np.arange(1e-7, 3e-6, 1e-9),Filter_vector=Filter_vector,filters_directory=filters_directory,Radius=None,distance_pc=None,output_plot=False,output_txt=False,z=z,Ebv=Ebv)[:,3])
	#pdb.set_trace()
	#print(Filter_vector)
	#pdb.set_trace()
	if z!=0:
		pylab.plot(spectrum_flux[:,0],
			   best_coeff1 * calc_black_body_flux_filters.calc_black_body_flux_filters(best_temp,np.arange(1e-7, 3e-6, 1e-9),Filter_vector=Filter_vector,filters_directory=filters_directory,Radius=None,distance_pc=None,output_plot=False,output_txt=False,z=z,Ebv=Ebv)[:,3]
,
			   'ro', label='synphot(best fit redshifted+extincted)')
	#else:
	#	print 'central wavelength is', central_wavelength
		#pdb.set_trace()
	#	pylab.plot(central_wavelength,best_coeff1 * calc_black_body_flux_filters.calc_black_body_flux_filters(best_temp,
	#																			wavelengths_in_meters * (z + 1),
	#																			Filter_vector, Radius=None,
	#																			distance_pc=None, output_plot=False,
	#																			output_txt=False)[:, 3],'go', label = 'best fit')
	#print('bb')
	bb=black_body_flux_density.black_body_flux_density(best_temp, np.arange(1e-7, 3e-6, 1e-9), 'P',
															   distance_pc=distance,
															   Radius=distances_conversions.cm_to_solar_radius(
																   best_radius),redshift=z,Ebv=Ebv)[2]



	#print('after bb')
	#pylab.figure()
	pylab.plot(bb[:,0]*1e10,bb[:,1],'-g', label='best fit redshifted+extincted')
	pylab.xlabel('wavelength $[\AA]$')
	pylab.ylabel('Flux density $[erg/cm^2/s/A]$')
	pylab.title('Spectrum (flux)')
	pylab.grid()
	pylab.legend(loc=1)
	pylab.savefig(output_file+'/fit_result_FLux.png', facecolor='w', edgecolor='w',
				  orientation='portrait', papertype=None, format='png', transparent=False, bbox_inches=None,
				  pad_inches=0.1)
	#pylab.show()
	'''
	if show_mag_AB==True:
		pylab.figure()
		pylab.plot(central_wavelength, mags[:], 'ro', label=r'data')
		if Ebv != 0:
			pylab.plot(central_wavelength, mags_correct[:], 'bo', label=r'data corrected for extinction')
		# This is in case there are 2 param in model l1*black_body+l2: pylab.plot(Spectrum_corrected[:,0]*1e9,best_coeff1*black_body_flux_density.black_body_flux_density(best_temp, Spectrum_corrected[:, 0], 'P')[0][:,1]+best_coeff2,'-b',label='best fit')
		#pylab.plot(central_wavelength,
		#		   best_coeff1 * calc_black_body_flux_filters.calc_black_body_flux_filters(best_temp,
		#																				   wavelengths_in_meters,
		#																				   Filter_vector, Radius=None,
		#																				   distance=None,
		#																				   output_plot=False,
		#																				   output_txt=False)[:, 3]
		#		   ,
		#		   'go', label='best fit')
		pylab.xlabel('wavelength $[\AA]$')
		pylab.ylabel('mag_AB')
		pylab.grid()
		pylab.title('Spectrum (mag)')
		pylab.legend(loc=1)
		pylab.savefig(output_file + '/fit_result_Mag.png', facecolor='w', edgecolor='w',
					  orientation='portrait', papertype=None, format='png', transparent=False, bbox_inches=None,
					  pad_inches=0.1)

	'''
	#Plot Xi_array fields
	f, axarr = pylab.subplots(3, sharex=True)
	#x_formatter = ScalarFormatter(useOffset=False)
	axarr[0].plot(Xi_array[:,0], Xi_array[:,2], 'b',
				  label=r'$\chi^2$')
	axarr[0].plot(best_temp, Xi_array[index_min, 2], 'ro',markersize=5)
	axarr[0].set_title(r'Results of fit')
	axarr[0].set_ylabel(r'best fit $\chi^2$')
	#axarr[1].set_xlabel(r'checked temperature')
	axarr[0].grid()
	axarr[0].axvline(x=best_temp,color='k',linestyle='--')
	if TempVec is None:
		axarr[0].set_xscale('log')
	axarr[1].plot(Xi_array[:,0], Xi_array[:,1],'r',
				  label=r'multiplicative factor')
	axarr[1].set_ylabel(r'best fit multiplicative factore')
	axarr[1].plot(best_temp, Xi_array[index_min, 1], 'ro',markersize=5)
	#pylab.xlim(min(julian_days_NUV), max(julian_days_NUV))
	#pylab.legend(label=['common measurement days'])
	# pylab.gca().invert_yaxis()
	axarr[1].grid()
	axarr[1].axvline(x=best_temp,color='k',linestyle='--')
	if TempVec is None:
		axarr[1].set_xscale('log')
	#axarr[0].xaxis.set_major_formatter(x_formatter)
	axarr[2].plot(Xi_array[:,0], Xi_array[:,3], 'm',
				  label=r'Radius')
	axarr[2].plot(best_temp, Xi_array[index_min, 3], 'ro',markersize=5)
	axarr[2].set_ylabel(r'best fit radius')
	axarr[2].set_xlabel(r'checked temperature')
	axarr[2].grid()
	axarr[2].axvline(x=best_temp,color='k',linestyle='--')
	if TempVec is None:
		axarr[2].set_xscale('log')
	#axarr[1].xaxis.set_major_formatter(x_formatter)
	pylab.savefig(output_file+'/fit_grid_results.png', facecolor='w', edgecolor='w', orientation='portrait',
				  papertype=None, format='png', transparent=False, bbox_inches=None, pad_inches=0.5)

	print('the best fit temperature is {0}'.format(best_temp))
	if show_plot == True:
		pylab.show()
	if distance is None:
		print('the best fit radius has been calculated for a distance of 10pc and is {0} cm ({1} solar radii)'.format(best_radius,distances_conversions.cm_to_solar_radius(best_radius)))
	else:
		print('the best fit radius has been calculated the provided distance of {0}pc and is {1} cm ({2} solar radii)'.format(distance,best_radius,distances_conversions.cm_to_solar_radius(best_radius)))
		print('the best fit coeff is {0}'.format(best_coeff1))
	result_array=np.zeros((1,6))
	result_array[0,0]=0
	result_array[0,1]=best_temp
	result_array[0,2]=index_min
	result_array[0,3]=best_coeff1
	result_array[0,4]=best_radius
	result_array[0,5]=best_luminosity
	#print result_array
	#pdb.set_trace()
	if path_to_txt_file  is not None:
		np.savetxt(path_to_txt_file,result_array,header='place_holder, best temperature, index, best coeff, best radius, best luminosity')
	else:
		np.savetxt(output_file+'/best_fit_results.txt',np.array(result_array),header='place_holder,best temperature, index, best coeff, best radius, best luminosity')
	return Xi_array,best_temp,index_min, best_coeff1, best_radius,best_luminosity



## Parameters used in script.py ##


mcmc=False# if False, plot with linear fit. If True, plots with mcmc

output_file_mcmc='./test/results_fit_sed_mcmc'
output_file_linear='./test/result_fit_sed_mat'
output_file_interpolation='./test/results_interpolation'
z =0
distance_modulus=33.51
explosion_date=0 #in principle, this has to be in jd. For the test data it is in jd-explosion dates
EBV=0.035
csm=False
data_file='./test/data_files/data_13dqy_formatted_for_package.txt' #must have at least the following fiels, and a header with the fields written as such: jd,flux,fluxerr,filter.
# filter can be one of the following: ('UVW1','UVW2','UVM2','u_swift','v_swift','b_swift','g_sdss','r_sdss','i_sdss','z_sdss'
# ,'r_cousin','i_cousin','h_2mass','j_2mass','u_johnson','b_johnson','v_johnson')
dates_file='./test/data_files/13dqy_int_dates.txt'

lower_limit_on_flux=1e-40
filters_directory='./PhotoFit/Filters' #don't touch this line

# Interpolation step
already_run_interp_errors=dict() #don't touch this line
already_run_interp_errors['UVW1']=False
already_run_interp_errors['UVW2']=False
already_run_interp_errors['UVM2']=False
already_run_interp_errors['u_swift']=False
already_run_interp_errors['b_swift']=False
already_run_interp_errors['v_swift']=False
already_run_interp_errors['r_sdss']=False
already_run_interp_errors['g_sdss']=False
already_run_interp_errors['i_sdss']=False
already_run_interp_errors['r_p48']=True
already_run_interp_errors['g_p48']=True
already_run_interp_errors['z_sdss']=True
already_run_interp_errors['u_johnson']=True
already_run_interp_errors['v_johnson']=True
already_run_interp_errors['b_johnson']=True
already_run_interp_errors['i_cousin']=True
already_run_interp_errors['r_cousin']=True
already_run_interp_errors['j_2mass']=True
already_run_interp_errors['h_2mass']=True


# In case you fit with a linear-fitting algorythm
already_run_matrix=False
num_iterations=100

# In case you fit with mcmc
already_run_mcmc=False
nwalkers=100
num_steps=350

# In case you want to compare your R and T results with existing results from a file
data_compare='./test/data_files/Yaron_2017_results.txt'#file with column#1: time from explosion, column#2: temperature (K), column#3:radius (cm)



#! /usr/bin/env python


import numpy as np
from . import class_chi2
import emcee
import pylab
import corner
import scipy.optimize as op
import os


# simple power lax ax^n or a(x-xref)^n
'''
def emcee_1_param(model_1param,prior_param1,data,uncertainties,initial_conditions,flatchain_path=None,already_run=False):
    """Description: given data, uncertainties, and priors on a and b, run emcee to fit the data with the model a(x-xref)^n (0 for x smaller than xref)
    Input  :-
    Output :-
    Tested : ?
         By : Maayane T. Soumagnac Nov 2016
        URL :
    Example:     Reliable:  """
    if initial_conditions[0] < np.min(prior_param1) or initial_conditions[0]>np.max(prior_param1):
        print('ERROR: you need to give initial condition for the one parameter which are within the prior range on a'
        exit()

    def lnprior(theta):
        param1 = theta
        if np.min(prior_param1) < param1 < np.max(prior_param1) :
            return 0.0
        return -np.inf

    def myloglike(theta):
        param1 = theta
        my_model = model_1param(param1, data[:,0]).model_array()
        my_objective = class_chi2.objective_with_uncertainties(my_model, data,
                                                                   sigmas=uncertainties)
        loglik = my_objective.chi_square_value()
        return -0.5 * loglik

    def lnprob(theta):
        lp = lnprior(theta)
        if not np.isfinite(lp):
            return -np.inf
        return lp + myloglike(theta)

    if already_run!=True:
        print('*** EMCEE run ***'
        ndim=1
        sampler = emcee.EnsembleSampler(nwalkers, ndim, lnprob)
        pos = np.zeros((nwalkers, ndim))
        print(pos)
        pos[:, 0] = [initial_conditions[0] + min(np.average(prior_param1),100.) * 1e-4*np.random.randn() for j in range(nwalkers)]

        #print(pos)
        #print('the min of pos is', np.min(pos[:, 2])
        #print('the min of pos is', np.max(pos[:, 2])
        #print('prior is', prior_xref
        if np.min(pos[:, 0]) <= np.min(prior_param1) or np.max(pos[:, 0]) >= np.max(prior_param1):
            print('ERROR! the pos is beyond the prior for the first param'
            exit()
        #pdb.set_trace()
        #pos = [[2.1,2.2]+ 1e-4*np.random.randn(ndim) for i in range(nwalkers)]
        pos, prob, state = sampler.run_mcmc(pos, 100)
        sampler.reset()
        sampler.run_mcmc(pos,1000)
        if flatchain_path==None:
            np.savetxt('./flatchain_fitter_powerlaw.txt',sampler.flatchain)
        else:
            np.savetxt(flatchain_path, sampler.flatchain)
        print("Mean acceptance fraction:", np.mean(sampler.acceptance_fraction))
#print("Autocorrelation time:", sampler.get_autocorr_time())
        samples = sampler.chain[:, 50:, :].reshape((-1, ndim))
    else:
        if flatchain_path==None:
            samples=np.genfromtxt('./flatchain_fitter_powerlaw.txt')
        else:
            samples = np.genfromtxt(flatchain_path)
    return samples
'''
def emcee_n_param(ndim,model_nparam, prior_param, data, uncertainties, initial_conditions, nwalkers=100,num_steps=200,flatchain_path=None,already_run=False):
    """Description: given data, uncertainties, and priors on a and b, run emcee to fit the data with the model a(x-xref)^n (0 for x smaller than xref)
    Input  :-model_nparam a class like in the models.py file, that has a model_array method
            -a tuple where every element is a 2-1 array with the prior
            -num_steps goes into sampler.run_mcmc(pos,num_steps), default is 200.
    Output :-
    Tested : ?
         By : Maayane T. Soumagnac Nov 2016
        URL :
    Example: samples=fitter_general.emcee_n_param(ndim=2,model_nparam=model_lo_to,prior_param=[prior_alpha,prior_tref],data=my_data,uncertainties=errors,initial_conditions=[alpha_true,tref_true],flatchain_path='output_test_fitter_general/flatchain_test.txt',already_run=False)
    Reliable:  """
    if np.shape(initial_conditions)[0]!=ndim:
        #print np.shape(initial_conditions)[0]
        print('you did not give initial conditions for every parameter')
        exit()
    for i, j in enumerate(initial_conditions):
        if j < np.min(prior_param[i]) or j > np.max(prior_param[i]):
            print('ERROR: you need to give initial condition for parameter number {0} which are within the prior range'.format(i+1))
            exit()

    def lnprior(theta):
        if ndim==1:
            param0 = theta
            if np.min(prior_param[0]) < param0 < np.max(prior_param[0]):
                return 0.0
        elif ndim==2:
            param0,param1=theta
            if np.min(prior_param[0]) < param0 < np.max(prior_param[0]) and np.min(prior_param[1]) < param1 < np.max(prior_param[1]):
                return 0.0
        elif ndim==3:
            param0,param1,param2=theta
            if np.min(prior_param[0]) < param0 < np.max(prior_param[0]) and np.min(prior_param[1]) < param1 < np.max(prior_param[1]) and np.min(prior_param[2]) < param2 < np.max(prior_param[2]):
                return 0.0
        elif ndim==4:
            param0,param1,param2,param3=theta
            if np.min(prior_param[0]) < param0 < np.max(prior_param[0]) and np.min(prior_param[1]) < param1 < np.max(prior_param[1]) and np.min(prior_param[2]) < param2 < np.max(prior_param[2]) and np.min(prior_param[3]) < param3 < np.max(prior_param[3]):
                return 0.0
        elif ndim==5:
            param0, param1, param2, param3,param4 = theta
            if np.min(prior_param[0]) < param0 < np.max(prior_param[0]) and np.min(prior_param[1]) < param1 < np.max(
                    prior_param[1]) and np.min(prior_param[2]) < param2 < np.max(prior_param[2]) and np.min(
                    prior_param[3]) < param3 < np.max(prior_param[3]) and np.min(
                    prior_param[4]) < param4 < np.max(prior_param[4]):
                return 0.0
        elif ndim==6:
            param0, param1, param2, param3,param4,param5 = theta
            if np.min(prior_param[0]) < param0 < np.max(prior_param[0]) and np.min(prior_param[1]) < param1 < np.max(
                    prior_param[1]) and np.min(prior_param[2]) < param2 < np.max(prior_param[2]) and np.min(
                    prior_param[3]) < param3 < np.max(prior_param[3]) and np.min(
                    prior_param[4]) < param4 < np.max(prior_param[4]) and np.min(
                    prior_param[5]) < param5 < np.max(prior_param[5]):
                return 0.0
        elif ndim==7:
            param0, param1, param2, param3,param4,param5,param6 = theta
            if np.min(prior_param[0]) < param0 < np.max(prior_param[0]) and np.min(prior_param[1]) < param1 < np.max(
                    prior_param[1]) and np.min(prior_param[2]) < param2 < np.max(prior_param[2]) and np.min(
                    prior_param[3]) < param3 < np.max(prior_param[3]) and np.min(
                    prior_param[4]) < param4 < np.max(prior_param[4]) and np.min(
                    prior_param[5]) < param5 < np.max(prior_param[5]) and np.min(
                    prior_param[6]) < param6 < np.max(prior_param[6]):
                return 0.0
        return -np.inf

    def myloglike(theta):
        if ndim==1:
            param1 = theta
            my_model = model_nparam(param1, data[:, 0]).model_array()
        elif ndim==2:
            param1,param2 = theta
            #print param1
            #print param2
            my_model = model_nparam(param1,param2, data[:, 0]).model_array()
        elif ndim==3:
            param1,param2,param3 = theta
            my_model = model_nparam(param1,param2,param3, data[:, 0]).model_array()
        elif ndim==4:
            param1,param2,param3,param4 = theta
            my_model = model_nparam(param1,param2,param3, param4,data[:, 0]).model_array()
        elif ndim==5:
            param1,param2,param3,param4,param5 = theta
            my_model = model_nparam(param1,param2,param3, param4,param5,data[:, 0]).model_array()
        elif ndim==6:
            param1,param2,param3,param4,param5,param6 = theta
            my_model = model_nparam(param1,param2,param3, param4,param5,param6,data[:, 0]).model_array()
        elif ndim == 7:
            param1, param2, param3, param4, param5, param6, param7 = theta
            my_model = model_nparam(param1, param2, param3, param4, param5, param6,param7,data[:, 0]).model_array()

        my_objective = class_chi2.objective_with_uncertainties(my_model, data,
                                                               sigmas=uncertainties)
        loglik = my_objective.chi_square_value()
        return -0.5 * loglik

    def lnprob(theta):
        lp = lnprior(theta)
        if not np.isfinite(lp):
            return -np.inf
        return lp + myloglike(theta)

    if already_run != True:
        print('*** EMCEE run ***')
        #ndim = 1
        sampler = emcee.EnsembleSampler(nwalkers, ndim, lnprob)
        pos = np.zeros((nwalkers, ndim))
        for i in range(ndim):
            #print min(np.average(prior_param[i]), 100.)
            #print initial_conditions[i]
            #print np.average(prior_param[i])
            #pos[:, i] = [initial_conditions[i] + (np.max(prior_param[i])-np.average(prior_param[i])) * 1e-1 * np.random.randn() for j in range(nwalkers)]
            pos[:, i] = [
                np.average(prior_param[i]) + (np.max(prior_param[i]) - np.min(prior_param[i])) * 1e-1 * np.random.randn()
                for j in range(nwalkers)]
            if np.min(pos[:, i]) <= np.min(prior_param[i]) or np.max(pos[:, i]) >= np.max(prior_param[i]):
                #print(pos)
                print('initial condition is',initial_conditions[i])
                print(' 0.5*(np.max(prior_param[i]) - np.min(prior_param[i])) * 1e-3  is', (np.max(prior_param[i]) - np.min(prior_param[i])) * 1e-3)
                print('the min of pos is',np.min(pos[:, i]))
                print('the max of pos is',np.max(pos[:, i]))
                print('the prior is',prior_param[i])
                print('ERROR! the pos is beyond the prior for the '+str(i+1)+'th param')
                exit()
        #print(pos)
        for i in range(ndim):
            print('the min of pos is', np.min(pos[:, i]))
            print('the max of pos is', np.max(pos[:, i]))
            print('the prior is', prior_param[i])
        #pos, prob, state = sampler.run_mcmc(pos, 100)
        #sampler.reset()
        sampler.run_mcmc(pos, num_steps)
        if flatchain_path == None:
            np.savetxt('./flatchain_fitter_general.txt', sampler.flatchain)
        else:
            np.savetxt(flatchain_path, sampler.flatchain)
        print("Mean acceptance fraction:", np.mean(sampler.acceptance_fraction))
        # print("Autocorrelation time:", sampler.get_autocorr_time())
        samples = sampler.chain[:, 50:, :].reshape((-1, ndim))
    else:
        if flatchain_path == None:
            samples = np.genfromtxt('./flatchain_fitter_general.txt')
        else:
            samples = np.genfromtxt(flatchain_path)
    return samples

def calc_best_fit_n_param(ndim,model_nparam,flatchain_path,data,uncertainties,winners=50,output_file_path=None,bounds=None,already_run_calc_all_chis=False,show_plots=True,dilution_factor=0):
    """Description:
        -Input  :
        -Output : a numpy array with [best params, chi2]
        Plots and txt files:
        txt files:
       pdf files:
        Tested : ?
             By : Maayane T. Soumagnac Nov 2016
            URL :
        Example: best=fitter_general.calc_best_fit_n_param()
        Reliable:  """
    print('*** Calculation of the maximum likelihood values ***')
    samples=np.genfromtxt(flatchain_path,delimiter=None,dtype=float)
    if output_file_path==None:
        output_file_path='.'
    file_path_to_all_chain_chis = output_file_path + '/all_param_and_chis_of_chain.txt'
    #### 1. calculate the chi2 of all the parameters combinations of the chain, store this in file_path_to_all_chain_chis/all_param_and_chis_of_chain.txt'
    if already_run_calc_all_chis != True:
        if dilution_factor==0:
            print('I am calculating the chi value for each combination of parameters in the chain')
            chis = np.zeros((np.shape(samples)[0], ndim+1))
            for i, line in enumerate(samples):
                # print('i am a line {0}'.format(i)
                if ndim==1:
                    my_objective = class_chi2.objective_with_uncertainties(model_nparam(line[0],data[:, 0]).model_array(), data, sigmas=uncertainties)
                elif ndim==2:
                    my_objective = class_chi2.objective_with_uncertainties(model_nparam(line[0], line[1],data[:, 0]).model_array(),
                                                                           data, sigmas=uncertainties)
                elif ndim==3:
                    my_objective = class_chi2.objective_with_uncertainties(model_nparam(line[0],line[1],line[2], data[:, 0]).model_array(),
                                                                           data, sigmas=uncertainties)
                elif ndim==4:
                    my_objective=class_chi2.objective_with_uncertainties(model_nparam(line[0],line[1],line[2], line[3],data[:, 0]).model_array(),
                                                                           data, sigmas=uncertainties)
                elif ndim==5:
                    my_objective=class_chi2.objective_with_uncertainties(model_nparam(line[0],line[1],line[2], line[3],line[4],data[:, 0]).model_array(),
                                                                           data, sigmas=uncertainties)
                elif ndim==6:
                    my_objective=class_chi2.objective_with_uncertainties(model_nparam(line[0],line[1],line[2], line[3],line[4],line[5],data[:, 0]).model_array(),
                                                                           data, sigmas=uncertainties)
                elif ndim==7:
                    my_objective=class_chi2.objective_with_uncertainties(model_nparam(line[0],line[1],line[2], line[3],line[4],line[5],line[6],data[:, 0]).model_array(),
                                                                           data, sigmas=uncertainties)
                chis[i, 0:ndim] = samples[i, 0:ndim]
                chis[i, ndim] = my_objective.chi_square_value()
        else:
            print('I am calculating the chi value for every {0} combination of parameters in the chain'.format(dilution_factor))
            samples_diluted=samples[::dilution_factor].copy()
            chis = np.zeros((np.shape(samples_diluted)[0], ndim + 1))
            for i, line in enumerate(samples_diluted):
                # print('i am a line {0}'.format(i)
                if ndim == 1:
                    my_objective = class_chi2.objective_with_uncertainties(
                        model_nparam(line[0], data[:, 0]).model_array(), data, sigmas=uncertainties)
                elif ndim == 2:
                    my_objective = class_chi2.objective_with_uncertainties(
                        model_nparam(line[0], line[1], data[:, 0]).model_array(),
                        data, sigmas=uncertainties)
                elif ndim == 3:
                    my_objective = class_chi2.objective_with_uncertainties(
                        model_nparam(line[0], line[1], line[2], data[:, 0]).model_array(),
                        data, sigmas=uncertainties)
                elif ndim == 4:
                    my_objective = class_chi2.objective_with_uncertainties(
                        model_nparam(line[0], line[1], line[2], line[3], data[:, 0]).model_array(),
                        data, sigmas=uncertainties)
                elif ndim == 5:
                    my_objective = class_chi2.objective_with_uncertainties(
                        model_nparam(line[0], line[1], line[2], line[3], line[4], data[:, 0]).model_array(),
                        data, sigmas=uncertainties)
                elif ndim == 6:
                    my_objective = class_chi2.objective_with_uncertainties(
                        model_nparam(line[0], line[1], line[2], line[3], line[4], line[5], data[:, 0]).model_array(),
                        data, sigmas=uncertainties)
                elif ndim == 7:
                    my_objective = class_chi2.objective_with_uncertainties(
                        model_nparam(line[0], line[1], line[2], line[3], line[4], line[5], line[6],
                                     data[:, 0]).model_array(),
                        data, sigmas=uncertainties)
                chis[i, 0:ndim] = samples[i, 0:ndim]
                chis[i, ndim] = my_objective.chi_square_value()
        np.savetxt(file_path_to_all_chain_chis, chis)
    else:
        chis = np.genfromtxt(file_path_to_all_chain_chis)
    #### 2. sort all the chis by increasing order
    param_and_chi_sorted = chis[np.argsort(chis[:, ndim])]
    #### 3. take the n=winners combinations with lowest chi2, and use them as initial point of the op.minimize algorythm
    result_op_min = np.zeros((winners, np.shape(param_and_chi_sorted)[1]))

    def lnlike(theta, data, uncert):
        if ndim==1:
            x= theta
            return -0.5 * class_chi2.objective_with_uncertainties(model_nparam(x,data[:, 0]).model_array(),
                                                              data, sigmas=uncert).chi_square_value()
        elif ndim==2:
            x,y= theta
            return -0.5 * class_chi2.objective_with_uncertainties(model_nparam(x,y,data[:, 0]).model_array(),
                                                              data, sigmas=uncert).chi_square_value()
        elif ndim==3:
            x,y,z = theta
            return -0.5 * class_chi2.objective_with_uncertainties(model_nparam(x,y,z, data[:, 0]).model_array(),
                                                              data, sigmas=uncert).chi_square_value()
        elif ndim==4:
            x,y,z,w = theta
            return -0.5 * class_chi2.objective_with_uncertainties(model_nparam(x,y,z,w, data[:, 0]).model_array(),
                                                              data, sigmas=uncert).chi_square_value()
        elif ndim==5:
            x,y,z,w,u = theta
            return -0.5 * class_chi2.objective_with_uncertainties(model_nparam(x,y,z,w,u, data[:, 0]).model_array(),
                                                              data, sigmas=uncert).chi_square_value()
        elif ndim==6:
            x,y,z,w,u,r = theta
            return -0.5 * class_chi2.objective_with_uncertainties(model_nparam(x,y,z,w,u,r, data[:, 0]).model_array(),
                                                              data, sigmas=uncert).chi_square_value()
        elif ndim==7:
            x,y,z,w,u,r,s = theta
            return -0.5 * class_chi2.objective_with_uncertainties(model_nparam(x,y,z,w,u,r,s, data[:, 0]).model_array(),
                                                              data, sigmas=uncert).chi_square_value()

    nll = lambda *args: -lnlike(*args) / 0.5
    bounds=[[np.min(bounds[i]),np.max(bounds[i])] for i in range(ndim)]
    #print bounds
    for i in range(winners):
        result_op_mini = op.minimize(nll, [param_and_chi_sorted[i, j] for j in range(ndim)],args=(data, uncertainties), bounds=bounds)
        besties = result_op_mini["x"]
        #print besties
        result_op_min[i, 0:np.shape(param_and_chi_sorted)[1] - 1] = besties
        if ndim==1:
            result_op_min[i, np.shape(param_and_chi_sorted)[1] - 1] = class_chi2.objective_with_uncertainties(
            model_nparam(besties[0],data[:, 0]).model_array(), data,
            sigmas=uncertainties).chi_square_value()
        elif ndim==2:
            result_op_min[i, np.shape(param_and_chi_sorted)[1] - 1] = class_chi2.objective_with_uncertainties(
            model_nparam(besties[0],besties[1],data[:, 0]).model_array(), data,
            sigmas=uncertainties).chi_square_value()
        elif ndim==3:
            result_op_min[i, np.shape(param_and_chi_sorted)[1] - 1] = class_chi2.objective_with_uncertainties(
            model_nparam(besties[0],besties[1],besties[2],data[:, 0]).model_array(), data,
            sigmas=uncertainties).chi_square_value()
        elif ndim==4:
            result_op_min[i, np.shape(param_and_chi_sorted)[1] - 1] = class_chi2.objective_with_uncertainties(
            model_nparam(besties[0],besties[1],besties[2],besties[3],data[:, 0]).model_array(), data,
            sigmas=uncertainties).chi_square_value()
        elif ndim==5:
            result_op_min[i, np.shape(param_and_chi_sorted)[1] - 1] = class_chi2.objective_with_uncertainties(
            model_nparam(besties[0],besties[1],besties[2],besties[3],besties[4],data[:, 0]).model_array(), data,
            sigmas=uncertainties).chi_square_value()
        elif ndim==6:
            result_op_min[i, np.shape(param_and_chi_sorted)[1] - 1] = class_chi2.objective_with_uncertainties(
            model_nparam(besties[0],besties[1],besties[2],besties[3],besties[4],besties[5],data[:, 0]).model_array(), data,
            sigmas=uncertainties).chi_square_value()
        elif ndim == 7:
            result_op_min[i, np.shape(param_and_chi_sorted)[1] - 1] = class_chi2.objective_with_uncertainties(
                model_nparam(besties[0], besties[1], besties[2], besties[3], besties[4], besties[5],besties[6],
                             data[:, 0]).model_array(), data,
                sigmas=uncertainties).chi_square_value()

    #### 4. sort and store the obtained best fit parameters and corresponding chi in file_path_to_all_chain_chis/winners_best_chis.txt
    best_param_and_chi_sorted = result_op_min[np.argsort(result_op_min[:, ndim])]
    # print best_param_and_chi_sorted
    np.savetxt(output_file_path + '/' + str(winners) + '_best_combination_and_chis.txt', best_param_and_chi_sorted)

    #### 5. find the three optimized best fit combinations with lowest chi2 and store them in best_com
    maxparam = np.zeros((np.shape(best_param_and_chi_sorted)))
    # print np.shape(maxparam)
    # pdb.set_trace()
    j = 0
    maxparam[0, :] = best_param_and_chi_sorted[0, 0:np.shape(best_param_and_chi_sorted)[1]]
    for i in range(winners - 1):
        # print('param number ', i
        # print best_param_and_chi_sorted[i, 0:np.shape(best_param_and_chi_sorted)[1] - 1]  # print the a and b of the line
        a = (
        best_param_and_chi_sorted[i + 1, 0:np.shape(best_param_and_chi_sorted)[1] - 1] != best_param_and_chi_sorted[
                                                                                          i,
                                                                                          0:np.shape(
                                                                                              best_param_and_chi_sorted)[
                                                                                                1] - 1])
        # print a
        # print a.all
        if a.all():  # if the param are not the same
            # print('the line {0} is not identical to the line {1}'.format(i, i + 1)
            j = j + 1
            maxparam[j, :] = best_param_and_chi_sorted[i + 1,
                             0:np.shape(best_param_and_chi_sorted)[1]]  # we add the line to the column
            # else:
            #    print('the line {0} is identical to the line {1}'.format(i, i + 1
    maxiparam_good_size = maxparam

    np.savetxt(output_file_path + '/best_optimized_combinations.txt', maxiparam_good_size,
               header='best parameters,chi^2')
    # np.savetxt(output_file_path+'/first_best_combintation.txt', maxiparam_good_size[0, 0:np.shape(param_and_chi_sorted)[1]],header='best a, best n, chi^2')
    # if np.shape(maxiparam_good_size)[1]>1:
    #    np.savetxt(output_file_path + '/second_best_combintation.txt', maxparam[1, 0:np.shape(param_and_chi_sorted)[1]],
    #           header='#best a, best n, chi^2')
    # if np.shape(maxiparam_good_size)[1]>2:
    #    np.savetxt(output_file_path + '/third_best_combintation.txt', maxparam[2, 0:np.shape(param_and_chi_sorted)[1]],
    #           header='#best a, best n, chi^2')
    if show_plots == True:
        pylab.figure()
        pylab.plot(param_and_chi_sorted[:, ndim], 'b', label=r'$\chi^2 of the combinations in the chain$')
        pylab.title(r'sorted $\chi^2$ values in the chain {0}'.format(flatchain_path))
        pylab.xlabel('index')
        pylab.ylabel('$\chi^2$')
        pylab.savefig(output_file_path + '/all_chis_plot.pdf', facecolor='w', edgecolor='w', orientation='portrait',
                      papertype=None, format='pdf',
                      transparent=False, bbox_inches=None, pad_inches=0.1)
        pylab.figure()
        pylab.plot(best_param_and_chi_sorted[:, ndim], 'r')
        pylab.title(r'sorted optimized $\chi^2$ values')
        pylab.xlabel('index')
        pylab.ylabel('$\chi^2$')
        pylab.savefig(output_file_path + '/opt_chis_plot.pdf', facecolor='w', edgecolor='w', orientation='portrait',
                      papertype=None, format='pdf',
                      transparent=False, bbox_inches=None, pad_inches=0.1)
    print(maxiparam_good_size)
    print('the best fit param are {0} and the corrisponding chi^2 is {1}'.format(maxiparam_good_size[0,:ndim],maxiparam_good_size[0, ndim]))
    print('the reduced chi^2 is {0}'.format(round(maxiparam_good_size[0, ndim] / (np.shape(data)[0] - ndim), 4)))
    #print('the denominator is',(np.shape(data)[0] - ndim)
    #print ndim
    #print np.shape(data)[0]
    #pdb.set_trace()
    return maxiparam_good_size[0, :]
    #pylab.show()

def calc_best_fit_n_param_fast(ndim,flatchain_path,output_file_path=None):
    """Description:
        -Input  :
        -Output : a numpy array with [best params, chi2]
        Plots and txt files:
        txt files:
       pdf files:
        Tested : ?
             By : Maayane T. Soumagnac Nov 2016
            URL :
        Example: best=fitter_general.calc_best_fit_n_param()
        Reliable:  """
    print('*** Calculation of the maximum likelihood values ***')
    samples=np.genfromtxt(flatchain_path,delimiter=None,dtype=float)
    if output_file_path==None:
        output_file_path='.'
    bests=np.zeros(np.shape(samples)[1])
    file_path_to_all_chain_chis = output_file_path + '/all_param_and_chis_of_chain.txt'
    #### 1. calculate the chi2 of all the parameters combinations of the chain, store this in file_path_to_all_chain_chis/all_param_and_chis_of_chain.txt'
    for i in range(ndim):
        bests[i]=np.median(samples[:,i])
    np.savetxt(output_file_path + '/' +'best_from_median.txt',bests)
    return bests

def plot_opt_fit_n_param(ndim, model_nparam,bests,data,flatchain_path=None,uncertainties=None, output_file_path='.',xlabel=None,ylabel=None):
    """Description:
        Input  :
        Output : 
        Plots and txt files:
            - 
        Tested : ?
             By : Maayane T. Soumagnac Nov 2016
            URL :
        Example:
        Reliable:  """

    print('*** Plots of the best fit solution ***')
    if ndim==1:
        best_fit=model_nparam(bests[0],data[:,0]).model_array()
        emcee_fit = model_nparam(bests[0],np.linspace(np.min(data[:, 0]), np.max(data[:, 0]),
                                                              1000)).model_array()
    elif ndim==2:
        best_fit=model_nparam(bests[0],bests[1],data[:,0]).model_array()
        #print np.linspace(np.min(data[:, 0]), np.max(data[:, 0]),1000)
        emcee_fit = model_nparam(bests[0], bests[1],np.linspace(np.min(data[:, 0]), np.max(data[:, 0]),
                                                              1000)).model_array()
    elif ndim==3:
        best_fit=model_nparam(bests[0],bests[1],bests[2],data[:,0]).model_array()
        emcee_fit = model_nparam(bests[0], bests[1], bests[2],np.linspace(np.min(data[:, 0]), np.max(data[:, 0]),
                                                              1000)).model_array()
    elif ndim==4:
        best_fit=model_nparam(bests[0],bests[1],bests[2],bests[3],data[:,0]).model_array()
        emcee_fit = model_nparam(bests[0], bests[1], bests[2],bests[3],np.linspace(np.min(data[:, 0]), np.max(data[:, 0]),
                                                              1000)).model_array()
    elif ndim==5:
        best_fit=model_nparam(bests[0],bests[1],bests[2],bests[3],bests[4],data[:,0]).model_array()
        emcee_fit = model_nparam(bests[0], bests[1], bests[2],bests[3],bests[4],np.linspace(np.min(data[:, 0]), np.max(data[:, 0]),
                                                              1000)).model_array()
    elif ndim==6:
        best_fit=model_nparam(bests[0],bests[1],bests[2],bests[3],bests[4],bests[5],data[:,0]).model_array()
        emcee_fit = model_nparam(bests[0], bests[1], bests[2],bests[3],bests[4],bests[5],np.linspace(np.min(data[:, 0]), np.max(data[:, 0]),
                                                              1000)).model_array()
    elif ndim==7:
        best_fit=model_nparam(bests[0],bests[1],bests[2],bests[3],bests[4],bests[5],bests[6],data[:,0]).model_array()
        emcee_fit = model_nparam(bests[0], bests[1], bests[2],bests[3],bests[4],bests[5],bests[6],np.linspace(np.min(data[:, 0]), np.max(data[:, 0]),
                                                              1000)).model_array()
    elif ndim==8:
        best_fit=model_nparam(bests[0],bests[1],bests[2],bests[3],bests[4],bests[5],bests[6],bests[7],data[:,0]).model_array()
        emcee_fit = model_nparam(bests[0], bests[1], bests[2],bests[3],bests[4],bests[5],bests[6],bests[7],np.linspace(np.min(data[:, 0]), np.max(data[:, 0]),
                                                              1000)).model_array()
    elif ndim==9:
        best_fit=model_nparam(bests[0],bests[1],bests[2],bests[3],bests[4],bests[5],bests[6],bests[7],bests[8],data[:,0]).model_array()
        emcee_fit = model_nparam(bests[0], bests[1], bests[2],bests[3],bests[4],bests[5],bests[6],bests[7],bests[8],np.linspace(np.min(data[:, 0]), np.max(data[:, 0]),
                                                              1000)).model_array()
    fig = pylab.figure()
    pylab.plot(best_fit[:,0],best_fit[:,1], 'bo', label=r'best-fit model')
    #pylab.plot(emcee_fit[:,0],emcee_fit[:,1],'b-')
    if uncertainties is None==False:
        pylab.errorbar(data[:,0],data[:,1],yerr=0.5 * uncertainties, fmt='r.', label=r'data')
    else:
        pylab.plot(data[:, 0], data[:, 1],'r.', label=r'data')
    #pylab.title(r'Data and best-fit model, $\chi^2/dof={0}$'.format(round(class_chi2.objective_with_uncertainties(best_fit,data,uncertainties).chi_square_value() / (np.shape(data)[0] - ndim), 3)))
    ax = pylab.gca()
    ax.set_xlabel(xlabel)
    ax.set_ylabel(ylabel)
    pylab.legend(loc=1)
    my_objective=class_chi2.objective_with_uncertainties(best_fit,data, sigmas=uncertainties)
    chi=my_objective.chi_square_value()
    dof=np.shape(data)[0]-ndim
    pylab.title(r'Best fit, $\chi=$'+str(chi)+', $\chi/dof=$'+str(chi/dof))
    pylab.savefig(output_file_path + '/best_fit.pdf', facecolor='w', edgecolor='w',
                  orientation='portrait', papertype=None, format='pdf', transparent=False, bbox_inches=None,
                  pad_inches=0.1)

    if flatchain_path!=None:
        samples = np.genfromtxt(flatchain_path, comments='#')
        fig = pylab.figure()
        if ndim==1:
            for a in samples[np.random.randint(len(samples), size=15)]:
                emcee_fit_indata=model_nparam(a,np.linspace(np.min(data[:,0]),np.max(data[:,0]),1000))
                pylab.plot(emcee_fit_indata.model_array()[:, 0], emcee_fit_indata.model_array()[:, 1], 'b-', alpha=0.5)
        elif ndim==2:
            for a, b, in samples[np.random.randint(len(samples), size=15)]:
                emcee_fit_indata = model_nparam(a, b, data[:,0])
                emcee_fit_indata = model_nparam(a, b,np.linspace(np.min(data[:, 0]), np.max(data[:, 0]),
                                                                                 1000))
                pylab.plot(emcee_fit_indata.model_array()[:, 0], emcee_fit_indata.model_array()[:, 1], 'b-', alpha=0.5)
        elif ndim==3:
            for a,b,c in samples[np.random.randint(len(samples), size=15)]:
                emcee_fit_indata=model_nparam(a,b,c,np.linspace(np.min(data[:,0]),np.max(data[:,0]),1000))
                pylab.plot(emcee_fit_indata.model_array()[:, 0], emcee_fit_indata.model_array()[:, 1], 'b-', alpha=0.5)
        elif ndim==4:
            for a,b,c,d in samples[np.random.randint(len(samples), size=15)]:
                emcee_fit_indata=model_nparam(a,b,c,d,np.linspace(np.min(data[:,0]),np.max(data[:,0]),1000))
                pylab.plot(emcee_fit_indata.model_array()[:, 0], emcee_fit_indata.model_array()[:, 1], 'b-', alpha=0.5)
        elif ndim==5:
            for a,b,c,d,e in samples[np.random.randint(len(samples), size=15)]:
                emcee_fit_indata=model_nparam(a,b,c,d,e,np.linspace(np.min(data[:,0]),np.max(data[:,0]),1000))
                pylab.plot(emcee_fit_indata.model_array()[:, 0], emcee_fit_indata.model_array()[:, 1], 'b-', alpha=0.5)
        elif ndim == 6:
            for a, b, c, d, e,f in samples[np.random.randint(len(samples), size=15)]:
                emcee_fit_indata = model_nparam(a, b, c, d, e,f,
                                                np.linspace(np.min(data[:, 0]), np.max(data[:, 0]), 1000))
                pylab.plot(emcee_fit_indata.model_array()[:, 0], emcee_fit_indata.model_array()[:, 1], 'b-', alpha=0.5)
        elif ndim == 7:
            for a, b, c, d, e,f,g in samples[np.random.randint(len(samples), size=15)]:
                emcee_fit_indata = model_nparam(a, b, c, d, e,f,g,
                                                np.linspace(np.min(data[:, 0]), np.max(data[:, 0]), 1000))
                pylab.plot(emcee_fit_indata.model_array()[:, 0], emcee_fit_indata.model_array()[:, 1], 'b-', alpha=0.5)
        elif ndim == 8:
            for a, b, c, d, e,f,g,h in samples[np.random.randint(len(samples), size=15)]:
                emcee_fit_indata = model_nparam(a, b, c, d, e,f,g,h,
                                                np.linspace(np.min(data[:, 0]), np.max(data[:, 0]), 1000))
                pylab.plot(emcee_fit_indata.model_array()[:, 0], emcee_fit_indata.model_array()[:, 1], 'b-', alpha=0.5)
        elif ndim == 9:
            for a, b, c, d, e,f,g,h,k in samples[np.random.randint(len(samples), size=15)]:
                emcee_fit_indata = model_nparam(a, b, c, d, e,f,g,h,k,
                                                np.linspace(np.min(data[:, 0]), np.max(data[:, 0]), 1000))
                pylab.plot(emcee_fit_indata.model_array()[:, 0], emcee_fit_indata.model_array()[:, 1], 'b-', alpha=0.5)

        pylab.plot(best_fit[:, 0], best_fit[:, 1], 'ro', label=r'Best-fit model')
        #pylab.plot(emcee_fit[:, 0], emcee_fit[:, 1], 'r-')

        #uncomment
        '''
        if uncertainties != 'None':
            pylab.errorbar(data[:, 0], data[:, 1], yerr=0.5 * uncertainties, fmt='r.', label=r'data')
        else:
            pylab.plot(data[:, 0], data[:, 1], fmt='r.', label=r'data')
        '''
        #pylab.title(r'Data and best-fit model, $\chi^2/dof={0}$'.format(round(
        #    class_chi2.objective_with_uncertainties(best_fit, data, uncertainties).chi_square_value() / (
        #    np.shape(data)[0] - ndim), 3)))
        ax = pylab.gca()
        ax.set_xlabel(xlabel)
        ax.set_ylabel(ylabel)
        pylab.legend(loc=1)
        pylab.savefig(output_file_path + '/best_fit_with_chain.pdf', facecolor='w', edgecolor='w',
                  orientation='portrait', papertype=None, format='pdf', transparent=False, bbox_inches=None,
                  pad_inches=0.1)
    #pylab.show()

def plot_2D_distributions(flatchain_path,bests=None,output_file_path='.',parameters_labels=None,title=None):
    """Description: Given a chain, plots the 2-D marginalized distributions.
    Input  :-flatchain_Path: the path to the chain.
            -bests (optional):a numpy array with values of the parameters to be shown on the plots (ex: calculated best-fit values, true known values, etc)
            -output_file_path (optional): the path to the outputs folders. Default is '.'
            -parameters_labels (optional): a tuple with the names of the parameters to be added to the plot. Default is ['param 1','param 2']
    Output :- no output. plots.
    pdf files:
             - 2D-distributions.png : a corner plot with the 2D marginalized distributions.
    Tested : ?
         By : Maayane T. Soumagnac Nov 2016
        URL :
    Example: samples=fitter_powerlaw.fitter_powerlaw(np.array([0.,5.]),np.array([0.,5.]),my_data,uncertainties=errors*np.ones(50),flatchain_path='./flatchain_2_test.txt',already_run=True)
    Reliable:  """
    print('*** Plots of the 2D distributions ***')
    samples=np.genfromtxt(flatchain_path)
    #if output_file_path==None:
    #    output_file_path='.'
    #if parameters_labels==None:
    #    labels=['param 1','param 2']
    #else:
    #    labels=parameters_labels
        #print labels
    if title!=None:
        big_title=title
    if bests is None:
        #fig = triangle.corner(samples, labels=parameters_labels,
         #                quantiles=[0.159, 0.5, 0.841],show_titles=False,big_title=big_title)
        fig = corner.corner(samples, labels=parameters_labels,
                         quantiles=[0.159, 0.5, 0.841],show_titles=False,big_title=big_title)
        for ax in fig.get_axes():
        #    ax.tick_params(axis='both', labelsize=14)
            #ax.set_xlabel(fontsize=14)
            ax.xaxis.label.set_size(20)
            ax.yaxis.label.set_size(20)
    else:
        #fig = triangle.corner(samples,labels=parameters_labels,
        #                      show_titles=False,
        #                      truths=bests[:],quantiles=[0.159, 0.5, 0.841],big_title=big_title
        fig = corner.corner(samples,labels=parameters_labels,
                              show_titles=False,
                              truths=bests[:],quantiles=[0.159, 0.5, 0.841],big_title=big_title)
        for ax in fig.get_axes():
            #ax.tick_params(axis='both', labelsize=14)
            #ax.set_xlabel(fontsize=14)
            ax.yaxis.label.set_size(20)
            ax.xaxis.label.set_size(20)
            #ax.get_ylabel().set_fontsize(60)
    if os.path.isdir(output_file_path)==True:
        pylab.tight_layout()
        fig.savefig(output_file_path+'/2D-distributions.png')
    else:
        print('output_file_path is', output_file_path)
        pylab.tight_layout()
        fig.savefig(output_file_path)

def plot_1D_marginalized_distribution(flatchain_path, bests=None,output_pdf_file_path='.',output_txt_file_path='.', parameters_labels=None,number_bins=None):
        """Description: Given a chain, plots the 1-D marginalized distributions.
        Input  :-flatchain_Path: the path to the chain.
                -bests (optional):a numpy array with values of the parameters to be shown on the plots (ex: calculated best-fit values, true known values, etc)
                -output_file_path (optional): the path to the outputs folders. Default is '.'
                -parameters_labels (optional): a tuple with the names of the parameters to be added as xlabels. Default is Nonex
                -number of bins in the histograms. Default is 100
        Output :- no output. plots.
        pdf files:
                 - histo_param_' + parameters_labels(j) + 'pdf' histograms.
        Tested : ?
             By : Maayane T. Soumagnac Nov 2016
            URL :
        Example: histos=fitter_powerlaw.plot_1D_marginalized_distribution(flatchain_path='flatchain_2_test.txt',bests=bests,output_file_path='./output_test_fitter_powerlaw',parameters_labels=['a','b'],number_bins=50)
        Reliable:  """
        #if output_file_path == None:
        #    output_file_path = '.'
        #if number_bins == None:
        #    number_bins = 100
        print('*** Plots of the 1D distributions ***')
        if isinstance(flatchain_path,np.ndarray)==True:
            print('you gave flatchain_path as a numpy array')
            if len(np.shape(flatchain_path)) < 2:
                print('1D array')
                print(np.shape(flatchain_path))
                sample=np.zeros((np.shape(flatchain_path)[0],1))
                sample[:,0]=flatchain_path
            else:
                sample=flatchain_path
            print (np.shape(sample))
            print(sample)
        elif isinstance(flatchain_path,str):
            print('you gave flatchain_path as an actual path')
            sample = np.loadtxt(flatchain_path)
        if bests is None:
            sigma = np.zeros((np.shape(sample)[1], 4))
        else:
            sigma = np.zeros((np.shape(sample)[1], 5))
        for j in range(np.shape(sample)[1]):
            if parameters_labels is None:
                print('I am plotting the histogram for the parameter number {0}'.format(j))
            else:
                print('I am plotting the histogram for the parameter {0}'.format(parameters_labels[j]))

            pylab.figure()
            n, bin, patches = pylab.hist(sample[:, j], bins=number_bins, alpha=0.5, color='orange')
            #               pylab.axvline(bn_mcmc2, color='r', linestyle='dashed', linewidth=3,label=r'$\chi^2={0}$'.format(maxparam[1,np.shape(maxparam)[1]-1]))
            #               pylab.axvline(bn_mcmc3, color='b', linestyle='dashed', linewidth=3,label=r'$\chi^2={0}$'.format(maxparam[2,np.shape(maxparam)[1]-1]))
            #               pylab.legend()
            bin2 = np.zeros((np.shape(n)))
            #print np.shape(bin)
            #pdb.set_trace()
            for i in range(np.shape(n)[0]):
                bin2[i] = bin[i] + (bin[i + 1] - bin[i]) / 2
            for i, b in enumerate(bin):
                if float(sum(n[:i])) / sum(n) >= 0.16:
                    #print np.shape(bin)
                    #print i
                    #print np.shape(n)
                    #print sum(n[:i])
                    #print bin2[i]
                    print('i is ',i)
                    print('bin2 is of shape',np.shape(bin2))
                    print('sun(n[:i]) is',sum(n[:i]))
                    nlower = bin[i]
                    print('at the {0}th bin, i.e. at b={1}, the ration float(sum(n[:i]))/sum(n) is {2} and the 16percent limit has been reached'.format(
                        i, bin2[i], float(sum(n[:i])) / sum(n)))
                    break
            for i, b in enumerate(bin):
                if float(sum(n[:i])) / sum(n) >= 0.84:
                    nupper = bin[i]
                    print('the rached ratio is {0}'.format(float(sum(n[:i])) / sum(n)))
                    #print('i is ',i
                    #print('bin2 is of shape',np.shape(bin2)
                    #print bin2[i]
                    #print('sun(n[:i]) is',sum(n[:i])
                    print('at the {0}th bin, i.e. at b={1}, the ration float(sum(n[:i]))/sum(n) is {2} and the 16percent upper limit has been reached'.format(
                        i, bin2[i], float(sum(n[:i])) / sum(n)))
                    break
            for i, b in enumerate(bin):
                if float(sum(n[:i])) / sum(n) >= 0.5:
                    nmedian = bin[i]
                    print('the rached ratio is {0}'.format(float(sum(n[:i])) / sum(n)))
                    #print('i is ',i
                    #print('bin2 is of shape',np.shape(bin2)
                    #print bin2[i]
                    #print('sun(n[:i]) is',sum(n[:i])
                    print('at the {0}th bin, i.e. at b={1}, the ration float(sum(n[:i]))/sum(n) is {2} and the 50percent upper limit has been reached'.format(
                        i, bin2[i], float(sum(n[:i])) / sum(n)))
                    break
            pylab.plot(bin2, n, color='k')
            if bests is not None:
                pylab.axvline(bests[j], color='k', linestyle='dashed', linewidth=3,label=r'Best fit')#. $\chi^2={0}$'.format(maxparam[0, np.shape(maxparam)[1] - 1]))
            pylab.axvline(nlower, color='b', linestyle='dashed', linewidth=3, label=r'$1\sigma$-range lower limit')
            pylab.axvline(nmedian, color='c', linestyle='dashed', linewidth=3, label=r'median')
            pylab.axvline(nupper, color='r', linestyle='dashed', linewidth=3, label=r'$1\sigma$-range upper limit')
            pylab.legend(fontsize = 'x-small')
            pylab.ylabel('Marginalized Posterior Distribution')
            if parameters_labels!=None:
                pylab.xlabel(parameters_labels[j])
                pylab.savefig(output_pdf_file_path + '/histo_param_' + parameters_labels[j] + '.pdf', facecolor='w', edgecolor='w',
                              orientation='portrait', papertype=None, format='pdf', transparent=False, bbox_inches=None,
                              pad_inches=0.1)
            else:
                pylab.savefig(output_pdf_file_path + '/histo_param_'+str(j)+'.pdf', facecolor='w', edgecolor='w',
                          orientation='portrait', papertype=None, format='pdf', transparent=False, bbox_inches=None,
                          pad_inches=0.1)
            if bests is None:
                sigma[j, :] = nlower, nupper, float(nupper - nlower) / 2,nmedian
                np.savetxt(output_txt_file_path + '/1sigma.txt', sigma, header='lower 1sigma, upper 1sigma, diff/2,median')
            else:
                sigma[j, :] = nlower, nupper, float(nupper - nlower) / 2, nmedian,bests[j]
                np.savetxt(output_txt_file_path + '/1sigma.txt', sigma, header='lower 1sigma, upper 1sigma, diff/2,median, best-fit')


        #pylab.show()